@@ -225,11 +225,13 @@ namespace Smuxi.Engine
             TextMessagePartModel fmsgti;
 
             fmsgti = new TextMessagePartModel();
-            fmsgti.Text = _("[MsnProtocolManager Commands]");
+            // TRANSLATOR: this line is used as a label / category for a
+            // list of commands below
+            fmsgti.Text = "[" + _("MSN Commands") + "]";
             fmsgti.Bold = true;
             fmsg.MessageParts.Add(fmsgti);
             
-                Session.AddMessageToChat(cd.FrontendManager.CurrentChat, fmsg);
+            Session.AddMessageToChat(cd.Chat, fmsg);
             
             string[] help = {
             "help",
@@ -237,7 +239,7 @@ namespace Smuxi.Engine
             };
             
             foreach (string line in help) { 
-                cd.FrontendManager.AddTextToCurrentChat("-!- " + line);
+                cd.FrontendManager.AddTextToChat(cd.Chat, "-!- " + line);
             }
         }
         
