@@ -235,11 +235,13 @@ namespace Smuxi.Engine
             TextMessagePartModel fmsgti;
 
             fmsgti = new TextMessagePartModel();
-            fmsgti.Text = _("[OscarProtocolManager Commands]");
+            // TRANSLATOR: this line is used as a label / category for a
+            // list of commands below. {0} would be AIM or ICQ for example.
+            fmsgti.Text = "[" + String.Format(_("{0} Commands"), NetworkID) + "]";
             fmsgti.Bold = true;
             fmsg.MessageParts.Add(fmsgti);
             
-            this.Session.AddMessageToChat(cd.FrontendManager.CurrentChat, fmsg);
+            this.Session.AddMessageToChat(cd.Chat, fmsg);
             
             string[] help = {
             "help",
@@ -247,7 +249,7 @@ namespace Smuxi.Engine
             };
             
             foreach (string line in help) { 
-                cd.FrontendManager.AddTextToCurrentChat("-!- " + line);
+                cd.FrontendManager.AddTextToChat(cd.Chat, "-!- " + line);
             }
         }
         
