@@ -22,6 +22,7 @@
 
 using System;
 using System.Net;
+using System.Web;
 using System.Threading;
 using System.Collections.Generic;
 using Twitterizer.Framework;
@@ -98,7 +99,7 @@ namespace Smuxi.Engine
 
             f_FriendsTimelineChat = new GroupChatModel(
                 TwitterChatType.FriendsTimeline.ToString(),
-                _("Friends Timeline"),
+                _("Home Timeline"),
                 this
             );
             f_GroupChats.Add(f_FriendsTimelineChat);
@@ -161,40 +162,40 @@ namespace Smuxi.Engine
             // twitter is sometimes pretty slow, so fetch this in the background
             ThreadPool.QueueUserWorkItem(delegate {
                 try {
-                    msg =_("Fetching user details from Twitter, please wait...");
-                    Session.AddTextToChat(f_ProtocolChat, "-!- " + msg);
+                    var message = _("Fetching user details from Twitter, please wait...");
+                    Session.AddTextToChat(f_ProtocolChat, "-!- " + message);
 
                     UpdateUser();
 
-                    msg =_("Finished fetching user details.");
-                    Session.AddTextToChat(f_ProtocolChat, "-!- " + msg);
+                    message = _("Finished fetching user details.");
+                    Session.AddTextToChat(f_ProtocolChat, "-!- " + message);
 
                     f_FriendsTimelineChat.PersonCount = f_TwitterUser.NumberOfFriends;
                     f_RepliesChat.PersonCount = f_TwitterUser.NumberOfFriends;
                     f_DirectMessagesChat.PersonCount = f_TwitterUser.NumberOfFriends;
                 } catch (Exception ex) {
-                    msg =_("Failed to fetch user details from Twitter. Reason: ");
+                    var message = _("Failed to fetch user details from Twitter. Reason: ");
 #if LOG4NET
-                    f_Logger.Error("Connect(): " + msg, ex);
+                    f_Logger.Error("Connect(): " + message, ex);
 #endif
-                    Session.AddTextToChat(f_ProtocolChat, "-!- " + msg + ex.Message);
+                    Session.AddTextToChat(f_ProtocolChat, "-!- " + message + ex.Message);
                 }
             });
             ThreadPool.QueueUserWorkItem(delegate {
                 try {
-                    msg =_("Fetching friends from Twitter, please wait...");
-                    Session.AddTextToChat(f_ProtocolChat, "-!- " + msg);
+                    var message = _("Fetching friends from Twitter, please wait...");
+                    Session.AddTextToChat(f_ProtocolChat, "-!- " + message);
 
                     UpdateFriends();
 
-                    msg =_("Finished fetching friends.");
-                    Session.AddTextToChat(f_ProtocolChat, "-!- " + msg);
+                    message = _("Finished fetching friends.");
+                    Session.AddTextToChat(f_ProtocolChat, "-!- " + message);
                 } catch (Exception ex) {
-                    msg =_("Failed to fetch friends from Twitter. Reason: ");
+                    var message = _("Failed to fetch friends from Twitter. Reason: ");
 #if LOG4NET
-                    f_Logger.Error("Connect(): " + msg, ex);
+                    f_Logger.Error("Connect(): " + message, ex);
 #endif
-                    Session.AddTextToChat(f_ProtocolChat, "-!- " + msg + ex.Message);
+                    Session.AddTextToChat(f_ProtocolChat, "-!- " + message + ex.Message);
                 }
             });
 
@@ -259,6 +260,11 @@ namespace Smuxi.Engine
                 return;
             }
 
+            if (f_UpdateFriendsTimelineThread != null &&
+                f_UpdateFriendsTimelineThread.IsAlive) {
+                return;
+            }
+
             // BUG: causes a race condition as the frontend syncs the
             // unpopulated chat! So only add it if it's ready
             //Session.AddChat(f_FriendsTimelineChat);
@@ -283,6 +289,11 @@ namespace Smuxi.Engine
                 return;
             }
 
+            if (f_UpdateRepliesThread != null &&
+                f_UpdateRepliesThread.IsAlive) {
+                return;
+            }
+
             // BUG: causes a race condition as the frontend syncs the
             // unpopulated chat! So only add it if it's ready
             //Session.AddChat(f_RepliesChat);
@@ -307,6 +318,11 @@ namespace Smuxi.Engine
                 return;
             }
 
+            if (f_UpdateDirectMessagesThread != null &&
+                f_UpdateDirectMessagesThread.IsAlive) {
+                return;
+            }
+
             // BUG: causes a race condition as the frontend syncs the
             // unpopulated chat! So only add it if it's ready
             //Session.AddChat(f_DirectMessagesChat);
@@ -319,12 +335,12 @@ namespace Smuxi.Engine
             f_UpdateDirectMessagesThread.Start();
         }
 
-        private void OpenPrivateChat(int userId)
+        private ChatModel OpenPrivateChat(int userId)
         {
-            OpenPrivateChat(userId.ToString());
+            return OpenPrivateChat(userId.ToString());
         }
 
-        private void OpenPrivateChat(string userId)
+        private ChatModel OpenPrivateChat(string userId)
         {
             ChatModel chat =  Session.GetChat(
                 userId,
@@ -333,7 +349,7 @@ namespace Smuxi.Engine
             );
 
             if (chat != null) {
-                return;
+                return chat;
             }
 
             TwitterUser user = f_Twitter.User.Show(userId);
@@ -352,12 +368,38 @@ namespace Smuxi.Engine
             );
             Session.AddChat(personChat);
             Session.SyncChat(personChat);
+            return personChat;
         }
 
         public override void CloseChat(FrontendManager fm, ChatModel chat)
         {
             Trace.Call(fm, chat);
 
+            if (chat.ChatType == ChatType.Group) {
+               TwitterChatType twitterChatType = (TwitterChatType)
+                    Enum.Parse(typeof(TwitterChatType), chat.ID);
+               switch (twitterChatType) {
+                    case TwitterChatType.FriendsTimeline:
+                        if (f_UpdateFriendsTimelineThread != null &&
+                            f_UpdateFriendsTimelineThread.IsAlive) {
+                            f_UpdateFriendsTimelineThread.Abort();
+                        }
+                        break;
+                    case TwitterChatType.Replies:
+                        if (f_UpdateRepliesThread != null &&
+                            f_UpdateRepliesThread.IsAlive) {
+                            f_UpdateRepliesThread.Abort();
+                        }
+                        break;
+                    case TwitterChatType.DirectMessages:
+                        if (f_UpdateDirectMessagesThread != null &&
+                            f_UpdateDirectMessagesThread.IsAlive) {
+                            f_UpdateDirectMessagesThread.Abort();
+                        }
+                        break;
+                }
+            }
+
             Session.RemoveChat(chat);
         }
 
@@ -456,7 +498,14 @@ namespace Smuxi.Engine
                 switch (twitterChatType) {
                     case TwitterChatType.FriendsTimeline:
                     case TwitterChatType.Replies:
-                        PostUpdate(cmd.Data);
+                        try {
+                            PostUpdate(cmd.Data);
+                        } catch (Exception ex) {
+                            fm.AddTextToChat(cmd.Chat, "-!- " +
+                                String.Format(_("Could not update status - Reason: {0}"),
+                                              ex.Message)
+                            );
+                        }
                         break;
                     case TwitterChatType.DirectMessages:
                         fm.AddTextToChat(
@@ -468,7 +517,17 @@ namespace Smuxi.Engine
                         break;
                 }
             } else if (cmd.Chat.ChatType == ChatType.Person) {
-                SendMessage(cmd.Chat.ID, cmd.Data);
+                try {
+                    SendMessage(cmd.Chat.ID, cmd.Data);
+                } catch (Exception ex) {
+#if LOG4NET
+                    f_Logger.Error(ex);
+#endif
+                    fm.AddTextToChat(cmd.Chat, "-!- " +
+                        String.Format(_("Could not send message - Reason: {0}"),
+                                      ex.Message)
+                    );
+                }
             } else {
                 // ignore protocol chat
             }
@@ -499,11 +558,18 @@ namespace Smuxi.Engine
                 return;
             }
 
-            OpenPrivateChat(user.ID);
+            var chat = OpenPrivateChat(user.ID);
 
             if (cmd.DataArray.Length >= 3) {
                 string message = String.Join(" ", cmd.DataArray, 2, cmd.DataArray.Length-2);
-                SendMessage(user.ID.ToString(), message);
+                try {
+                    SendMessage(user.ID.ToString(), message);
+                } catch (Exception ex) {
+                    fm.AddTextToChat(chat, "-!- " +
+                        String.Format(_("Could not send message - Reason: {0}"),
+                                      ex.Message)
+                    );
+                }
             }
          }
 
@@ -565,10 +631,17 @@ namespace Smuxi.Engine
 #endif
                 string msg =_("An error occurred while fetching the friends timeline from Twitter. Reason: ");
                 Session.AddTextToChat(f_ProtocolChat, "-!- " + msg + ex.Message);
-            }
+            } finally {
 #if LOG4NET
-            f_Logger.Debug("UpdateFriendsTimelineThread(): finishing thread.");
+                f_Logger.Debug("UpdateFriendsTimelineThread(): finishing thread.");
 #endif
+                lock (Session.Chats) {
+                    if (Session.Chats.Contains(f_FriendsTimelineChat)) {
+                        Session.RemoveChat(f_FriendsTimelineChat);
+                    }
+                }
+                f_FriendsTimelineChat.UnsafePersons.Clear();
+            }
         }
 
         private void UpdateFriendsTimeline()
@@ -585,7 +658,7 @@ namespace Smuxi.Engine
                                f_LastFriendsTimelineStatusID);
             }
             TwitterStatusCollection timeline =
-                f_Twitter.Status.FriendsTimeline(parameters);
+                f_Twitter.Status.HomeTimeline(parameters);
 #if LOG4NET
             f_Logger.Debug("UpdateFriendsTimeline(): done. New tweets: " +
                 (timeline == null ? 0 : timeline.Count));
@@ -648,10 +721,17 @@ namespace Smuxi.Engine
 #endif
                 string msg =_("An error occurred while fetching the replies from Twitter. Reason: ");
                 Session.AddTextToChat(f_ProtocolChat, "-!- " + msg + ex.Message);
-            }
+            } finally {
 #if LOG4NET
-            f_Logger.Debug("UpdateRepliesThread(): finishing thread.");
+                f_Logger.Debug("UpdateRepliesThread(): finishing thread.");
 #endif
+                lock (Session.Chats) {
+                    if (Session.Chats.Contains(f_RepliesChat)) {
+                        Session.RemoveChat(f_RepliesChat);
+                    }
+                }
+                f_RepliesChat.UnsafePersons.Clear();
+            }
         }
 
         private void UpdateReplies()
@@ -736,10 +816,17 @@ namespace Smuxi.Engine
 #endif
                 string msg =_("An error occurred while fetching direct messages from Twitter. Reason: ");
                 Session.AddTextToChat(f_ProtocolChat, "-!- " + msg + ex.Message);
-            }
+            } finally {
 #if LOG4NET
-            f_Logger.Debug("UpdateDirectMessagesThread(): finishing thread.");
+                f_Logger.Debug("UpdateDirectMessagesThread(): finishing thread.");
 #endif
+                lock (Session.Chats) {
+                    if (Session.Chats.Contains(f_DirectMessagesChat)) {
+                        Session.RemoveChat(f_DirectMessagesChat);
+                    }
+                }
+                f_DirectMessagesChat.UnsafePersons.Clear();
+            }
         }
 
         private void UpdateDirectMessages()
@@ -925,7 +1012,7 @@ namespace Smuxi.Engine
             msgPart.Text = "> ";
             msg.MessageParts.Add(msgPart);
 
-            msgPart = new TextMessagePartModel(message);
+            msgPart = new TextMessagePartModel(HttpUtility.HtmlDecode(message));
             msgPart.IsHighlight = highlight;
             msg.MessageParts.Add(msgPart);
 
@@ -950,9 +1037,17 @@ namespace Smuxi.Engine
         {
             Trace.Call(exception == null ? null : exception.GetType());
 
-            if (exception.InnerException is WebException) {
+            if (exception.RequestData != null &&
+                exception.RequestData.ResponseException != null) {
+                CheckWebException(exception.RequestData.ResponseException);
+                return;
+            } else if (exception.InnerException is WebException) {
                 CheckWebException((WebException) exception.InnerException);
                 return;
+            } else if (exception.InnerException != null) {
+#if LOG4NET
+                f_Logger.Warn("CheckTwitterizerException(): unknown inner exception: " + exception.InnerException.GetType(), exception.InnerException);
+#endif
             }
 
             throw exception;
@@ -964,7 +1059,11 @@ namespace Smuxi.Engine
 
             switch (exception.Status) {
                 case WebExceptionStatus.ConnectFailure:
+                case WebExceptionStatus.ConnectionClosed:
+                case WebExceptionStatus.Timeout:
                 case WebExceptionStatus.ReceiveFailure:
+                case WebExceptionStatus.NameResolutionFailure:
+                case WebExceptionStatus.ProxyNameResolutionFailure:
                     // ignore temporarly issues
 #if LOG4NET
                     f_Logger.Warn("CheckWebException(): ignored exception", exception);
@@ -1006,6 +1105,7 @@ namespace Smuxi.Engine
                 case HttpStatusCode.BadRequest:
                 case HttpStatusCode.Forbidden:
                 case HttpStatusCode.ServiceUnavailable:
+                case HttpStatusCode.GatewayTimeout:
                     // ignore temporarly issues
 #if LOG4NET
                     f_Logger.Warn("CheckWebException(): ignored exception", exception);
