@@ -56,7 +56,7 @@ namespace Smuxi.Engine
 #endif
         protected bool          m_IsCleanConfig;
         protected Hashtable     m_Preferences = Hashtable.Synchronized(new Hashtable());
-        public event EventHandler Changed;
+        public event EventHandler<ConfigChangedEventArgs> Changed;
         
         public object this[string key] {
             get {
@@ -69,9 +69,14 @@ namespace Smuxi.Engine
 #endif
                     return;
                 }
+                var oldValue = m_Preferences[key];
                 m_Preferences[key] = value;
-                if (Changed != null) {
-                    Changed(this, EventArgs.Empty);
+
+                // only raise event if the value changed
+                if (!value.Equals(oldValue)) {
+                    if (Changed != null) {
+                        Changed(this, new ConfigChangedEventArgs(key, value));
+                    }
                 }
             }
         }
@@ -123,12 +128,16 @@ namespace Smuxi.Engine
                 // the section and key exist
                 string strValue = section.GetValue(inikey);
                 Type targetType = typeof(T);
-                if (String.IsNullOrEmpty(strValue)) {
-                    return default(T);
+                if (targetType == typeof(string)) {
+                    return (T)(object) strValue;
                 }
                 if (targetType == typeof(string[])) {
                     return (T)(object) GetList(key);
                 }
+                // handle empty booleans and integers
+                if (targetType.IsValueType && String.IsNullOrEmpty(strValue)) {
+                    return default(T);
+                }
 
                 return (T) Convert.ChangeType(strValue, targetType);
             }
@@ -177,7 +186,7 @@ namespace Smuxi.Engine
             
 #elif CONFIG_NINI
             // Nini does not support native string lists, have to emulate them
-            string result_str = (string)Get(key, null);
+            string result_str = Get<string>(key, null);
             if (result_str != null) {
                 if (result_str.Length > 0) {
                     result = result_str.Split('|');
@@ -271,12 +280,16 @@ namespace Smuxi.Engine
             prefix = "Engine/Users/DEFAULT/Connection/";
             Get(prefix+"Encoding", String.Empty);
             
+            prefix = "Engine/Users/DEFAULT/Logging/";
+            Get(prefix+"Enabled", false);
+            Get(prefix+"LogFilteredMessages", false);
+
             prefix = "Engine/Users/DEFAULT/Servers/";
             Get(prefix + "Servers", new string[] {
                 "IRC/irc.oftc.net",
                 "IRC/irc.gimp.org",
                 "IRC/irc.efnet.org",
-                "IRC/irc.ircnet.com",
+                "IRC/irc.ircnet.org",
                 "IRC/irc.freenode.net"
             });
             
@@ -284,6 +297,8 @@ namespace Smuxi.Engine
             Get(prefix + "Hostname", "irc.oftc.net");
             Get(prefix + "Port", 6667);
             Get(prefix + "Network", "OFTC");
+            Get(prefix + "Username", String.Empty);
+            Get(prefix + "Password", String.Empty);
             Get(prefix + "OnStartupConnect", true);
             Get(prefix + "OnConnectCommands",
                 new string[] {
@@ -295,21 +310,29 @@ namespace Smuxi.Engine
             Get(prefix + "Hostname", "irc.gimp.org");
             Get(prefix + "Port", 6667);
             Get(prefix + "Network", "GIMPNet");
+            Get(prefix + "Username", String.Empty);
+            Get(prefix + "Password", String.Empty);
 
             prefix = "Engine/Users/DEFAULT/Servers/IRC/irc.efnet.org/";
             Get(prefix + "Hostname", "irc.efnet.org");
             Get(prefix + "Port", 6667);
             Get(prefix + "Network", "EFnet");
+            Get(prefix + "Username", String.Empty);
+            Get(prefix + "Password", String.Empty);
                 
             prefix = "Engine/Users/DEFAULT/Servers/IRC/irc.ircnet.org/";
             Get(prefix + "Hostname", "irc.ircnet.org");
             Get(prefix + "Port", 6667);
             Get(prefix + "Network", "IRCnet");
+            Get(prefix + "Username", String.Empty);
+            Get(prefix + "Password", String.Empty);
                 
             prefix = "Engine/Users/DEFAULT/Servers/IRC/irc.freenode.net/";
             Get(prefix + "Hostname", "irc.freenode.net");
             Get(prefix + "Port", 6667);
             Get(prefix + "Network", "freenode");
+            Get(prefix + "Username", String.Empty);
+            Get(prefix + "Password", String.Empty);
             
             prefix = "Engine/Users/";
             Get(prefix+"Users", new string[] { "local" });
@@ -385,6 +408,13 @@ namespace Smuxi.Engine
                     m_Preferences[prefix+user+"/Connection/OnConnectCommands"] = new string[] {};
                 }
                 
+                string[] highlight_words = GetList(prefix+user+"/Interface/Chat/HighlightWords");
+                if (highlight_words != null) {
+                    m_Preferences[prefix+user+"/Interface/Chat/HighlightWords"] = highlight_words;
+                } else {
+                    m_Preferences[prefix+user+"/Interface/Chat/HighlightWords"] = new string[] {};
+                }
+
                 LoadUserEntry(user, "Interface/Notebook/TimestampFormat", null);
                 LoadUserEntry(user, "Interface/Notebook/TabPosition", null);
                 LoadUserEntry(user, "Interface/Notebook/BufferLines", null);
@@ -412,6 +442,9 @@ namespace Smuxi.Engine
                 
                 LoadUserEntry(user, "Sound/BeepOnHighlight", null);
                 
+                LoadUserEntry(user, "Logging/Enabled", null);
+                LoadUserEntry(user, "Logging/LogFilteredMessages", null);
+
                 string[] servers = null;
                 string sprefix = prefix + user + "/Servers/";
                 servers = GetList(sprefix + "Servers");
@@ -466,21 +499,22 @@ namespace Smuxi.Engine
                     }
                 }
 
-                string[] channelFilters = null;
-                string cprefix = "Filters/Channel/";
-                channelFilters = GetList(prefix + user + "/" + cprefix + "Patterns");
-                if (channelFilters == null) {
-                    channelFilters = new string[] {};
-                    m_Preferences[prefix + user + "/" + cprefix + "Patterns"] = new string[] {};
+                string[] filters = null;
+                string cprefix = "Filters/";
+                filters = GetList(prefix + user + "/" + cprefix + "Filters");
+                if (filters == null) {
+                    filters = new string[] {};
+                    m_Preferences[prefix + user + "/" + cprefix + "Filters"] = new string[] {};
                 } else {
-                    m_Preferences[prefix + user + "/" + cprefix + "Patterns"] = channelFilters;
+                    m_Preferences[prefix + user + "/" + cprefix + "Filters"] = filters;
                 }
-                foreach (string filter in channelFilters) {
-                    cprefix = "Filters/Channel/" + filter + "/";
-                    LoadUserEntry(user, cprefix + "Pattern", null);
-                    LoadUserEntry(user, cprefix + "FilterJoins", null);
-                    LoadUserEntry(user, cprefix + "FilterParts", null);
-                    LoadUserEntry(user, cprefix + "FilterQuits", null);
+                foreach (string filter in filters) {
+                    cprefix = "Filters/" + filter + "/";
+                    LoadUserEntry(user, cprefix + "Protocol", null);
+                    LoadUserEntry(user, cprefix + "ChatType", null);
+                    LoadUserEntry(user, cprefix + "ChatID", null);
+                    LoadUserEntry(user, cprefix + "MessageType", null);
+                    LoadUserEntry(user, cprefix + "MessagePattern", null);
                 }
             }
         }
@@ -540,7 +574,7 @@ namespace Smuxi.Engine
 #endif
             
             if (Changed != null) {
-                Changed(this, EventArgs.Empty);
+                Changed(this, new ConfigChangedEventArgs(key, null));
             }
         }
 
@@ -628,4 +662,16 @@ namespace Smuxi.Engine
         }
 #endif
     }
+
+    public class ConfigChangedEventArgs : EventArgs
+    {
+        public string Key { get; private set; }
+        public object Value { get; private set; }
+
+        public ConfigChangedEventArgs(string key, object value)
+        {
+            Key = key;
+            Value = value;
+        }
+    }
 }
