@@ -27,6 +27,7 @@
  */
 
 using System;
+using System.Runtime.Remoting;
 using System.Collections;
 
 namespace Smuxi.Engine
@@ -41,6 +42,8 @@ namespace Smuxi.Engine
         private string    _DefaultPrefix = "Engine/Users/DEFAULT/";
         private Hashtable _Cache; 
         
+        public event EventHandler<ConfigChangedEventArgs> Changed;
+
         public bool IsCaching
         {
             get {
@@ -98,9 +101,26 @@ namespace Smuxi.Engine
         public UserConfig(Config config, string username)
         {
             _Config = config;
+            // HACK: The Changed event was introduced in 0.7.2, for backwards
+            // compatibility with 0.7.x server we need to suppress remoting
+            // exceptions here
+            try {
+                // we can't use events over remoting
+                if (!RemotingServices.IsTransparentProxy(config)) {
+                    _Config.Changed += OnConfigChanged;
+                }
+            } catch (Exception ex) {
+#if LOG4NET
+                _Logger.Warn(
+                    "UserConfig() registration of Config.Changed event failed, " +
+                    "ignoring for backwards compatibility with 0.7.x servers...",
+                    ex
+                );
+#endif
+            }
             _UserPrefix = "Engine/Users/"+username+"/";
         }
-        
+
         public void ClearCache()
         {
             if (IsCaching) {
@@ -128,5 +148,21 @@ namespace Smuxi.Engine
         {
             _Config.Save();
         }
+
+        void OnConfigChanged(object sender, ConfigChangedEventArgs e)
+        {
+            if (Changed == null) {
+                // no listeners
+                return;
+            }
+
+            if (!e.Key.StartsWith(_UserPrefix)) {
+                // setting for some other user has changed
+                return;
+            }
+
+            var key = e.Key.Substring(_UserPrefix.Length);
+            Changed(this, new ConfigChangedEventArgs(key, e.Value));
+        }
     }
 }
