@@ -21,9 +21,13 @@
  */
 
 using System;
+using System.IO;
+using System.Text;
+using System.Text.RegularExpressions;
 using System.Collections;
 using System.Collections.Generic;
 using System.Runtime.Remoting;
+using System.Threading;
 using Smuxi.Common;
 
 namespace Smuxi.Engine
@@ -43,6 +47,8 @@ namespace Smuxi.Engine
         private string                                _Username;
         private ProtocolManagerFactory                _ProtocolManagerFactory;
         private UserConfig                            _UserConfig;
+        private FilterListController                  _FilterListController;
+        private ICollection<FilterModel>              _Filters;
         private bool                                  _OnStartupCommandsProcessed;
         
         public IList<IProtocolManager> ProtocolManagers {
@@ -109,12 +115,15 @@ namespace Smuxi.Engine
             _FrontendManagers = new Dictionary<string, FrontendManager>();
             _ProtocolManagers = new List<IProtocolManager>();
             _UserConfig = new UserConfig(config, username);
+            _UserConfig.Changed += OnUserConfigChanged;
+            _FilterListController = new FilterListController(_UserConfig);
+            _Filters = _FilterListController.GetFilterList().Values;
             _Chats = new List<ChatModel>();
             
             _SessionChat = new SessionChatModel("smuxi", "smuxi");
             _Chats.Add(_SessionChat);
         }
-        
+
         public void RegisterFrontendUI(IFrontendUI ui)
         {
             Trace.Call(ui);
@@ -371,7 +380,8 @@ namespace Smuxi.Engine
                 }
             } else {
                 // normal text
-                if (cd.FrontendManager.CurrentProtocolManager == null) {
+                if (cd.Chat.ChatType == ChatType.Session &&
+                    cd.FrontendManager.CurrentProtocolManager == null) {
                     _NotConnected(cd);
                     handled = true;
                 }
@@ -479,16 +489,27 @@ namespace Smuxi.Engine
                                             cd.DataArray.Length - 1));
                 cd = new CommandModel(fm, cd.Chat, cd.CommandCharacter, cmd);
             }
-            protocolManager.Command(cd);
 
-            // set this as current protocol manager
-            // but only if there was none set (we might be on a chat for example)
-            // or if this is the neutral "smuxi" tab
-            if (fm.CurrentProtocolManager == null ||
-                fm.CurrentChat != null && fm.CurrentChat.ChatType == ChatType.Session) {
-                fm.CurrentProtocolManager = protocolManager;
-                fm.UpdateNetworkStatus();
-            }
+            // run in background so it can't block the command queue
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    protocolManager.Command(cd);
+
+                    // set this as current protocol manager
+                    // but only if there was none set (we might be on a chat for example)
+                    // or if this is the neutral "smuxi" tab
+                    if (fm.CurrentProtocolManager == null ||
+                        (fm.CurrentChat != null && fm.CurrentChat.ChatType == ChatType.Session)) {
+                        fm.CurrentProtocolManager = protocolManager;
+                        fm.UpdateNetworkStatus();
+                    }
+                } catch (Exception ex) {
+#if LOG4NET
+                    f_Logger.Error("CommandConnect(): ", ex);
+#endif
+                    fm.AddTextToChat(cd.Chat, "-!- " + _("Connect failed!"));
+                }
+            });
         }
         
         public void CommandDisconnect(CommandModel cd)
@@ -520,8 +541,12 @@ namespace Smuxi.Engine
                     )
                 );
             } else {
-                fm.CurrentProtocolManager.Disconnect(fm);
-                _ProtocolManagers.Remove(fm.CurrentProtocolManager);
+                var pm = cd.Chat.ProtocolManager;
+                if (pm == null) {
+                    return;
+                }
+                pm.Disconnect(fm);
+                _ProtocolManagers.Remove(pm);
             }
         }
         
@@ -533,8 +558,22 @@ namespace Smuxi.Engine
                 throw new ArgumentNullException("cd");
             }
             
-            FrontendManager fm = cd.FrontendManager;
-            fm.CurrentProtocolManager.Reconnect(fm);
+            var pm = cd.Chat.ProtocolManager;
+            if (pm == null) {
+                return;
+            }
+
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    pm.Reconnect(cd.FrontendManager);
+                } catch (Exception ex) {
+#if LOG4NET
+                    f_Logger.Error("CommandReconnect(): ", ex);
+#endif
+                    cd.FrontendManager.AddTextToChat(cd.Chat, "-!- " +
+                        _("Reconnect failed!"));
+                }
+            });
         }
         
         public void CommandConfig(CommandModel cd)
@@ -634,10 +673,10 @@ namespace Smuxi.Engine
                     return;
                 }
             } else if (cd.DataArray.Length >= 2) {
-                // current network manager
-                pm = fm.CurrentProtocolManager;
+                // network manager of chat
+                pm = cd.Chat.ProtocolManager;
             }
-            
+
             if (pm != null) {
                 pm.Disconnect(fm);
                 pm.Dispose();
@@ -735,7 +774,12 @@ namespace Smuxi.Engine
             }
             
             lock (_Chats) {
-                _Chats.Remove(chat);
+                if (!_Chats.Remove(chat)) {
+#if LOG4NET
+                    f_Logger.Warn("RemoveChat(): _Chats.Remove(" + chat + ") failed, ignoring...");
+#endif
+                    return;
+                }
             }
             
             lock (_FrontendManagers) {
@@ -793,6 +837,11 @@ namespace Smuxi.Engine
         }
         
         public void AddTextToChat(ChatModel chat, string text)
+        {
+            AddTextToChat(chat, text, false);
+        }
+
+        public void AddTextToChat(ChatModel chat, string text, bool ignoreFilters)
         {
             if (chat == null) {
                 throw new ArgumentNullException("chat");
@@ -801,10 +850,16 @@ namespace Smuxi.Engine
                 throw new ArgumentNullException("text");
             }
             
-            AddMessageToChat(chat, new MessageModel(text));
+            AddMessageToChat(chat, new MessageModel(text), ignoreFilters);
         }
         
         public void AddMessageToChat(ChatModel chat, MessageModel msg)
+        {
+            AddMessageToChat(chat, msg, false);
+        }
+
+        public void AddMessageToChat(ChatModel chat, MessageModel msg,
+                                     bool ignoreFilters)
         {
             if (chat == null) {
                 throw new ArgumentNullException("chat");
@@ -813,6 +868,12 @@ namespace Smuxi.Engine
                 throw new ArgumentNullException("msg");
             }
             
+            bool isFiltered = !ignoreFilters && IsFilteredMessage(chat, msg);
+            LogMessage(chat, msg, isFiltered);
+            if (isFiltered) {
+                return;
+            }
+
             int buffer_lines = (int) UserConfig["Interface/Notebook/EngineBufferLines"];
             if (buffer_lines > 0) {
                 chat.UnsafeMessages.Add(msg);
@@ -1029,6 +1090,132 @@ namespace Smuxi.Engine
             return _ProtocolManagerFactory.CreateProtocolManager(info, this);
         }
         
+        public void LogMessage(ChatModel chat, MessageModel msg, bool isFiltered)
+        {
+            if (chat == null) {
+                throw new ArgumentNullException("chat");
+            }
+            if (msg == null) {
+                throw new ArgumentNullException("msg");
+            }
+
+            if (!(bool) UserConfig["Logging/Enabled"]) {
+                return;
+            }
+            if (isFiltered && !(bool) UserConfig["Logging/LogFilteredMessages"]) {
+                return;
+            }
+
+            if (chat.ChatType == ChatType.Session ||
+                chat.ChatType == ChatType.Protocol) {
+                return;
+            }
+
+            try {
+                var logPath = Platform.LogPath;
+                var protocol = chat.ProtocolManager.Protocol.ToLower();
+                // HACK: twitter retrieves older messages and we don't want to
+                // re-log those when the twitter connection is re-opened
+                if (protocol == "twitter") {
+                    return;
+                }
+                var network = chat.ProtocolManager.NetworkID.ToLower();
+                logPath = Path.Combine(logPath, protocol);
+                if (network != protocol) {
+                    logPath = Path.Combine(logPath, network);
+                }
+                if (!Directory.Exists(logPath)) {
+                    Directory.CreateDirectory(logPath);
+                }
+                var chatId = chat.ID.Replace(" ", "_").ToLower();
+                logPath = Path.Combine(logPath, String.Format("{0}.log", chatId));
+                using (var stream = File.AppendText(logPath)) {
+                    stream.WriteLine(
+                        String.Format(
+                            "[{0:yyyy-MM-dd HH:mm:ss}] {1}",
+                            msg.TimeStamp.ToLocalTime(),
+                            msg.ToString()
+                        )
+                    );
+                }
+            } catch (Exception ex) {
+#if LOG4NET
+                f_Logger.Error("LogMessage(): logging error", ex);
+#endif
+            }
+        }
+
+        public bool IsFilteredMessage(ChatModel chat, MessageModel msg)
+        {
+            if (chat == null) {
+                throw new ArgumentNullException("chat");
+            }
+            if (msg == null) {
+                throw new ArgumentNullException("msg");
+            }
+
+            return IsFilteredMessage(chat, msg.ToString(), msg.MessageType);
+        }
+
+        public bool IsFilteredMessage(ChatModel chat, string msg,
+                                      MessageType msgType)
+        {
+            if (chat == null) {
+                throw new ArgumentNullException("chat");
+            }
+            if (msg == null) {
+                throw new ArgumentNullException("msg");
+            }
+
+            lock (_Filters) {
+                foreach (var filter in _Filters) {
+                    if (!String.IsNullOrEmpty(filter.Protocol) &&
+                        chat.ProtocolManager != null &&
+                        filter.Protocol != chat.ProtocolManager.Protocol) {
+                        continue;
+                    }
+                    if (filter.ChatType.HasValue &&
+                        filter.ChatType != chat.ChatType) {
+                        continue;
+                    }
+                    if (!String.IsNullOrEmpty(filter.ChatID) &&
+                        !Pattern.IsMatch(chat.ID, filter.ChatID)) {
+                        continue;
+                    }
+                    if (filter.MessageType.HasValue &&
+                        filter.MessageType != msgType) {
+                        continue;
+                    }
+                    if (!String.IsNullOrEmpty(filter.MessagePattern)) {
+                        var pattern = filter.MessagePattern;
+                        if (!Pattern.ContainsPatternCharacters(pattern)) {
+                            // use globbing by default
+                            pattern = String.Format("*{0}*", pattern);
+                        }
+                        if (!Pattern.IsMatch(msg, pattern)) {
+                            continue;
+                        }
+                    }
+
+                    return true;
+                }
+            }
+
+            return false;
+        }
+
+       void OnUserConfigChanged(object sender, ConfigChangedEventArgs e)
+       {
+            if (e.Key.StartsWith("Filters/")) {
+#if LOG4NET
+                f_Logger.Debug("OnUserConfigChanged(): refreshing filters");
+#endif
+                // referesh filters
+                // TODO: use a timeout here to only refresh once in 1 second
+                _Filters = _FilterListController.GetFilterList().Values;
+            }
+       }
+
         private static string _(string msg)
         {
             return LibraryCatalog.GetString(msg, _LibraryTextDomain);
