@@ -5,7 +5,7 @@
  * $Author: meebey $
  * $Date: 2007-04-17 23:17:37 +0200 (Tue, 17 Apr 2007) $
  *
- * smuxi - Smart MUltipleXed Irc
+ * Smuxi - Smart MUltipleXed Irc
  *
  * Copyright (c) 2005-2006 Mirco Bauer <meebey@meebey.net>
  *
@@ -29,11 +29,7 @@
 using System.Reflection;
 using System.Runtime.CompilerServices;
 
-#if UI_GTK
-[assembly: AssemblyTitle("smuxi - IRC support for GTK+ frontend")]
-#elif UI_GNOME
-[assembly: AssemblyTitle("smuxi - IRC support for GNOME frontend")]
-#endif
+[assembly: AssemblyTitle("Smuxi - IRC support for GNOME frontend")]
 [assembly: AssemblyCopyright("2005-2008 (C) Mirco Bauer <meebey@meebey.net>")]
 
 [assembly: AssemblyDelaySign(false)]
