@@ -51,7 +51,7 @@ namespace Smuxi.Frontend.Gnome
         private static SplashScreenWindow _SplashScreenWindow;
         private static MainWindow         _MainWindow;
 #if GTK_SHARP_2_10
-        private static Gtk.StatusIcon     _StatusIcon;
+        private static StatusIconManager  _StatusIconManager;
 #endif
         private static FrontendConfig     _FrontendConfig;
         private static Session            _LocalSession;
@@ -60,6 +60,8 @@ namespace Smuxi.Frontend.Gnome
         private static FrontendManager    _FrontendManager;
         private static TaskQueue          _FrontendManagerCheckerQueue;
         private static object             _UnhandledExceptionSyncRoot = new Object();
+        private static bool               _InCrashHandler;
+        private static bool               _InReconnectHandler;
 
         public static event EventHandler  SessionPropertyChanged;
 
@@ -107,15 +109,7 @@ namespace Smuxi.Frontend.Gnome
                 return _MainWindow;
             }
         }
-        
-#if GTK_SHARP_2_10
-        public static Gtk.StatusIcon StatusIcon {
-            get {
-                return _StatusIcon;
-            }
-        }
-#endif
-        
+
         public static Session Session {
             get {
                 return _Session;
@@ -128,7 +122,13 @@ namespace Smuxi.Frontend.Gnome
                 }
             }
         }
-        
+
+        public static bool IsLocalEngine {
+            get {
+                return _Session == _LocalSession;
+            }
+        }
+
         public static FrontendManager FrontendManager {
             get {
                 return _FrontendManager;
@@ -213,32 +213,6 @@ namespace Smuxi.Frontend.Gnome
  
             _MainWindow = new MainWindow();
 
-#if GTK_SHARP_2_10
-            _StatusIcon = new Gtk.StatusIcon();
-            _StatusIcon.Visible = false;
-            _StatusIcon.Pixbuf = new Gdk.Pixbuf(null, "icon.svg");
-            _StatusIcon.Activate += delegate {
-                try {
-                    if (_StatusIcon.Blinking) {
-                        _MainWindow.Present();
-                        return;
-                    }
-                    // not everyone uses a window list applet thus we have to
-                    // restore from minimized state here, see:
-                    // http://projects.qnetp.net/issues/show/159
-                    if (_MainWindow.IsMinimized) {
-                        _MainWindow.Present();
-                        return;
-                    }
-                    _MainWindow.Visible = !_MainWindow.Visible;
-                } catch (Exception ex) {
-                    ShowException(ex);
-                }
-            };
-            _StatusIcon.PopupMenu += OnStatusIconPopupMenu;
-            _StatusIcon.Tooltip = "Smuxi";
-#endif
-            
             if (String.IsNullOrEmpty((string) FrontendConfig["Engines/Default"])) {
                 InitLocalEngine();
                 ConnectEngineToGUI();
@@ -275,7 +249,7 @@ namespace Smuxi.Frontend.Gnome
         
         public static void ConnectEngineToGUI()
         {
-            if (_Session == _LocalSession) {
+            if (IsLocalEngine) {
                 // HACK: SessionManager.Register() is not used for local engines
                 _LocalSession.RegisterFrontendUI(_MainWindow.UI);
             }
@@ -335,7 +309,9 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call();
 
             try {
-                _FrontendManager.IsFrontendDisconnecting = true;
+                if (_FrontendManager != null) {
+                    _FrontendManager.IsFrontendDisconnecting = true;
+                }
                 _Session.DeregisterFrontendUI(_MainWindow.UI);
             } catch (System.Net.Sockets.SocketException ex) {
                 // ignore as the connection is maybe already broken
@@ -412,6 +388,9 @@ namespace Smuxi.Frontend.Gnome
             }
             
             if (ex != null) {
+#if LOG4NET
+                _Logger.Error("ShowError(): Exception: ", ex);
+#endif
                 msg += "\n" + String.Format(_("Cause: {0}"), ex.Message);
             }
             if (parent == null) {
@@ -436,6 +415,20 @@ namespace Smuxi.Frontend.Gnome
             ShowError(parent, msg, null);
         }
         
+        public static void ShowError(Gtk.Window parent, Exception ex)
+        {
+            Trace.Call(parent, ex != null ? ex.GetType() : null);
+
+            if (ex == null) {
+                throw new ArgumentNullException("ex");
+            }
+
+#if LOG4NET
+            _Logger.Error("ShowError(): Exception:", ex);
+#endif
+            ShowError(parent, ex.Message, null);
+        }
+
         public static void ShowException(Gtk.Window parent, Exception ex)
         {
             Trace.Call(parent, ex != null ? ex.GetType() : null);
@@ -460,6 +453,12 @@ namespace Smuxi.Frontend.Gnome
             // http://projects.qnetp.net/issues/show/232
             if (ex is System.Runtime.Remoting.RemotingException ||
                 ex is System.Net.Sockets.SocketException) {
+                if (_InReconnectHandler || _InCrashHandler) {
+                    // one reconnect is good enough and a crash we won't survive
+                    return;
+                }
+                _InReconnectHandler = true;
+
                 Gtk.MessageDialog md = new Gtk.MessageDialog(parent,
                     Gtk.DialogFlags.Modal, Gtk.MessageType.Error,
                     Gtk.ButtonsType.OkCancel, _("The frontend has lost the connection to the server.\nDo you want to reconnect now?"));
@@ -471,6 +470,7 @@ namespace Smuxi.Frontend.Gnome
                         try {
                             Frontend.ReconnectEngineToGUI();
                             // yay, we made it
+                            _InReconnectHandler = false;
                             break;
                         } catch (Exception e) {
 #if LOG4NET
@@ -500,7 +500,14 @@ namespace Smuxi.Frontend.Gnome
                 Quit();
                 return;
             }
-            
+
+            if (_InCrashHandler) {
+                // only show not more than one crash dialog, else the user
+                // will not be able to copy/paste the stack trace and stuff
+                return;
+            }
+            _InCrashHandler = true;
+
             CrashDialog cd = new CrashDialog(parent, ex);
             cd.Run();
             cd.Destroy();
@@ -532,64 +539,9 @@ namespace Smuxi.Frontend.Gnome
                 throw new ArgumentNullException("userConfig");
             }
             
-#if GTK_SHARP_2_10
-            string modeStr = (string) userConfig["Interface/Notification/NotificationAreaIconMode"];
-            NotificationAreaIconMode mode = (NotificationAreaIconMode) Enum.Parse(
-                typeof(NotificationAreaIconMode),
-                modeStr
-            );
-            switch (mode) {
-                case NotificationAreaIconMode.Never:
-                    _StatusIcon.Visible = false;
-                    break;
-                case NotificationAreaIconMode.Always:
-                    _StatusIcon.Visible = true;
-                    break;
-                case NotificationAreaIconMode.Minimized:
-                case NotificationAreaIconMode.Closed:
-                    // at application startup the main window is not realized but not visible
-                    _StatusIcon.Visible = _MainWindow.IsRealized && !_MainWindow.Visible;
-                    break;
-            }
-#endif
-            
             _MainWindow.ApplyConfig(userConfig);
         }
-        
-        private static void OnStatusIconPopupMenu(object sender, EventArgs e)
-        {
-            Trace.Call(sender, e);
-            
-            Gtk.Menu menu = new Gtk.Menu();
-            
-            Gtk.ImageMenuItem preferencesItem = new Gtk.ImageMenuItem(Gtk.Stock.Preferences, null);
-            preferencesItem.Activated += delegate {
-                try {
-                    PreferencesDialog dialog = new PreferencesDialog(_MainWindow);
-                    dialog.CurrentPage = PreferencesDialog.Page.Interface;
-                    dialog.CurrentInterfacePage = PreferencesDialog.InterfacePage.Notification;
-                } catch (Exception ex) {
-                    ShowException(ex);
-                }
-            };
-            menu.Add(preferencesItem);
-            
-            menu.Add(new Gtk.SeparatorMenuItem());
-            
-            Gtk.ImageMenuItem quitItem = new Gtk.ImageMenuItem(Gtk.Stock.Quit, null);
-            quitItem.Activated += delegate {
-                try {
-                    Quit();
-                } catch (Exception ex) {
-                    ShowException(ex);
-                }
-            };
-            menu.Add(quitItem);
-            
-            menu.ShowAll();
-            menu.Popup();
-        }
-        
+
 #if GTK_SHARP_2_10
         private static void _OnUnhandledException(GLib.UnhandledExceptionArgs e)
         {
