@@ -52,6 +52,9 @@ namespace Smuxi.Frontend.Gnome
         private EngineManager    _EngineManager;
         private Gtk.MenuItem     _CloseChatMenuItem;
         private NotificationAreaIconMode _NotificationAreaIconMode;
+#if GTK_SHARP_2_10
+        private StatusIconManager _StatusIconManager;
+#endif
         private bool             _IsMinimized;
         private bool             _IsMaximized;
         
@@ -120,7 +123,10 @@ namespace Smuxi.Frontend.Gnome
                 return _IsMinimized;
             }
         }
-        
+
+        public EventHandler Minimized;
+        public EventHandler Unminimized;
+
         public MainWindow() : base("Smuxi")
         {
             // restore window size / position
@@ -161,6 +167,7 @@ namespace Smuxi.Frontend.Gnome
             
             DeleteEvent += OnDeleteEvent;
             FocusInEvent += OnFocusInEvent;
+            FocusOutEvent += OnFocusOutEvent;
             WindowStateEvent += OnWindowStateEvent;
                     
             
@@ -354,6 +361,10 @@ namespace Smuxi.Frontend.Gnome
             _ChatViewManager.ChatAdded += OnChatViewManagerChatAdded;
             _ChatViewManager.ChatRemoved += OnChatViewManagerChatRemoved;
             
+#if GTK_SHARP_2_10
+            _StatusIconManager = new StatusIconManager(this, _ChatViewManager);
+#endif
+
             _UI = new GnomeUI(_ChatViewManager);
             
             // HACK: Frontend.FrontendConfig out of scope
@@ -402,11 +413,48 @@ namespace Smuxi.Frontend.Gnome
             );
             _NotificationAreaIconMode = mode;
             
+#if GTK_SHARP_2_10
+            _StatusIconManager.ApplyConfig(userConfig);
+#endif
             _Entry.ApplyConfig(userConfig);
             _Notebook.ApplyConfig(userConfig);
             _ChatViewManager.ApplyConfig(userConfig);
         }
-        
+
+        public void UpdateTitle()
+        {
+            UpdateTitle(null, null);
+        }
+
+        public void UpdateTitle(ChatView chatView, string protocolStatus)
+        {
+            Trace.Call(chatView, protocolStatus);
+
+            if (chatView == null) {
+                chatView = Notebook.CurrentChatView;
+            }
+            if (chatView == null) {
+                return;
+            }
+
+            string title;
+            if (chatView is SessionChatView) {
+                title = String.Empty;
+            } else if (chatView is ProtocolChatView) {
+                title = protocolStatus;
+            } else {
+                title = String.Format("{0} @ {1}",
+                                      chatView.Name,
+                                      protocolStatus);
+            }
+            if (!String.IsNullOrEmpty(title)) {
+                title += " - ";
+            }
+            title += "Smuxi";
+
+            Title = title;
+        }
+
         private void _OnQuitButtonClicked(object obj, EventArgs args)
         {
             Trace.Call(obj, args);
@@ -444,20 +492,20 @@ namespace Smuxi.Frontend.Gnome
             
             try {
                 UrgencyHint = false;
-#if GTK_SHARP_2_10
-                // HACK: out of scope?
-                Frontend.StatusIcon.Blinking = false;
-#endif
                 ChatView chatView = _Notebook.CurrentChatView;
                 if (chatView != null) {
                     // clear activity and highlight
                     chatView.HasHighlight = false;
                     chatView.HasActivity = false;
+                    var lastMsg = chatView.OutputMessageTextView.LastMessage;
                     // update last seen highlight
                     ThreadPool.QueueUserWorkItem(delegate {
                         try {
+                            if (lastMsg == null) {
+                                return;
+                            }
                             // REMOTING CALL 1
-                            chatView.ChatModel.LastSeenHighlight = DateTime.UtcNow;
+                            chatView.ChatModel.LastSeenHighlight = lastMsg.TimeStamp;
                         } catch (Exception ex) {
 #if LOG4NET
                             f_Logger.Error("OnFocusInEvent(): Exception", ex);
@@ -470,6 +518,18 @@ namespace Smuxi.Frontend.Gnome
             }
         }
         
+        protected virtual void OnFocusOutEvent(object sender, EventArgs e)
+        {
+            Trace.Call(sender, e);
+
+            try {
+                var chatView = _Notebook.CurrentChatView;
+                chatView.OutputMessageTextView.UpdateMarkerline();
+            } catch (Exception ex) {
+                Frontend.ShowException(this, ex);
+            }
+        }
+
         protected virtual void OnServerQuickConnectButtonClicked(object sender, EventArgs e)
         {
             Trace.Call(sender, e);
@@ -686,39 +746,31 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(sender, e);
             
             try {
-#if GTK_SHARP_2_10
                 // handle minimize / un-minimize
                 if ((e.Event.ChangedMask & Gdk.WindowState.Iconified) != 0) {
                     _IsMinimized = (e.Event.NewWindowState & Gdk.WindowState.Iconified) != 0;
-    #if LOG4NET
+#if LOG4NET
                     f_Logger.Debug("OnWindowStateEvent(): _IsMinimized: " + _IsMinimized);
-    #endif
+#endif
                     #if DISABLED
-                    if (_NotificationAreaIconMode == NotificationAreaIconMode.Minimized) {
-                        // BUG: metacity is not allowing us to use the minimize state
-                        // to hide and enable the notfication area icon as switching
-                        // to a different workspace sets WindowState.Iconified on all
-                        // windows, thus this code is disabled. For more details see:
-                        // http://projects.qnetp.net/issues/show/158
-                        Frontend.StatusIcon.Visible = _IsMinimized;
-                        if (isMinimized) {
-                            Hide();
+                    // BUG: metacity is not allowing us to use the minimize state
+                    // to hide and enable the notfication area icon as switching
+                    // to a different workspace sets WindowState.Iconified on all
+                    // windows, thus this code is disabled. For more details see:
+                    // http://projects.qnetp.net/issues/show/158
+                    Hide();
+                    #endif
+                    if (_IsMinimized) {
+                        if (Minimized != null) {
+                            Minimized(this, EventArgs.Empty);
+                        }
+                    } else {
+                        if (Unminimized != null) {
+                            Unminimized(this, EventArgs.Empty);
                         }
                     }
-                    #endif
                 }
 
-                // handle hide / show
-                if (_NotificationAreaIconMode == NotificationAreaIconMode.Closed &&
-                    (e.Event.ChangedMask & Gdk.WindowState.Withdrawn) != 0) {
-                    bool isHidden = (e.Event.NewWindowState & Gdk.WindowState.Withdrawn) != 0;
-    #if LOG4NET
-                    f_Logger.Debug("OnWindowStateEvent(): isHidden: " + isHidden);
-    #endif
-                    Frontend.StatusIcon.Visible = isHidden;
-                }
-#endif
-                
                 // handle maximize / un-maximize
                 if ((e.Event.ChangedMask & Gdk.WindowState.Maximized) != 0) {
                     _IsMaximized = (e.Event.NewWindowState & Gdk.WindowState.Maximized) != 0;
@@ -878,9 +930,6 @@ namespace Smuxi.Frontend.Gnome
             
             if (!HasToplevelFocus) {
                 UrgencyHint = true;
-#if GTK_SHARP_2_10
-                Frontend.StatusIcon.Blinking = true;
-#endif
             }
         }
 
