@@ -57,6 +57,7 @@ namespace Smuxi.Frontend.Gnome
 
             Scrollable = true;
             SwitchPage += OnSwitchPage;
+            SwitchPage += OnBeforeSwitchPage;
         }
         
         public void ApplyConfig(UserConfig userConfig)
@@ -159,6 +160,13 @@ namespace Smuxi.Frontend.Gnome
 #endif
         }
 
+        [GLib.ConnectBefore]
+        protected virtual void OnBeforeSwitchPage(object sender, Gtk.SwitchPageArgs e)
+        {
+            var chatView = CurrentChatView;
+            chatView.OutputMessageTextView.UpdateMarkerline();
+        }
+
         protected virtual void OnSwitchPage(object sender, Gtk.SwitchPageArgs e)
         {
             Trace.Call(sender, e);
@@ -174,6 +182,7 @@ namespace Smuxi.Frontend.Gnome
             // clear activity and highlight
             chatView.HasHighlight = false;
             chatView.HasActivity = false;
+            var lastMsg = chatView.OutputMessageTextView.LastMessage;
 
             var method = Trace.GetMethodBase();
             f_SwitchPageQueue.Queue(delegate {
@@ -195,27 +204,16 @@ namespace Smuxi.Frontend.Gnome
                     Frontend.FrontendManager.CurrentProtocolManager = nmanager;
                 }
 
-                // even when we have no network manager, we still want to update the state
+                // even when we have no network manager, we still want to update
+                // the network status and title
                 // REMOTING CALL 4
                 Frontend.FrontendManager.UpdateNetworkStatus();
 
-                // sync title
-                // REMOTING CALL 5
-                string networkStatus = nmanager != null ?
-                                            nmanager.ToString() + " / " :
-                                            String.Empty;
-                Gtk.Application.Invoke(delegate {
-                    if (Frontend.MainWindow == null) {
-                        return;
-                    }
-                    Frontend.MainWindow.Title = String.Format("{0}{1} - Smuxi",
-                                                              networkStatus,
-                                                              chatView.Name);
-                });
-
                 // update last seen highlight
-                // REMOTING CALL 6
-                chatModel.LastSeenHighlight = DateTime.UtcNow;
+                // REMOTING CALL 5
+                if (lastMsg != null) {
+                    chatModel.LastSeenHighlight = lastMsg.TimeStamp;
+                }
 
                 stop = DateTime.UtcNow;
 #if LOG4NET
