@@ -28,11 +28,14 @@
 
 using System;
 using System.Text;
+using System.Threading;
 using System.Collections;
 using System.Globalization;
+using Process = System.Diagnostics.Process;
 using Smuxi;
 using Smuxi.Common;
 using Smuxi.Engine;
+using System.Text.RegularExpressions;
 
 namespace Smuxi.Frontend.Gnome
 {
@@ -43,6 +46,7 @@ namespace Smuxi.Frontend.Gnome
             Interface,
             Servers,
             Filters,
+            Logging,
         }
         
         public enum InterfacePage : int {
@@ -66,9 +70,11 @@ namespace Smuxi.Frontend.Gnome
         private Gtk.Notebook _InterfaceNotebook;
         [Glade.Widget("MenuTreeView")]
         private Gtk.TreeView _MenuTreeView;
+        [Glade.Widget("FilterListEventBox")]
+        private Gtk.EventBox _FilterListEventBox;
 #endregion
         
-        private ChannelFilterListView _ChannelFilterListView;
+        private FilterListWidget _FilterListWidget;
         private ServerListView _ServerListView;
         
         public Page CurrentPage {
@@ -131,6 +137,22 @@ namespace Smuxi.Frontend.Gnome
             ((Gtk.CheckButton)_Glade["OverrideBackgroundColorCheckButton"]).Toggled += OnOverrideBackgroundColorCheckButtonToggled;
             ((Gtk.CheckButton)_Glade["OverrideFontCheckButton"]).Toggled += OnOverrideFontCheckButtonToggled;
 
+            ((Gtk.TextView)_Glade["HighlightWordsTextView"]).Buffer.Changed += _OnChanged;
+            if (Frontend.EngineVersion < new Version("0.7.2")) {
+                // feature introduced in >= 0.7.2
+                ((Gtk.TextView)_Glade["HighlightWordsTextView"]).Sensitive = false;
+            }
+
+            ((Gtk.Button)_Glade["LoggingOpenButton"]).Clicked += delegate {
+                ThreadPool.QueueUserWorkItem(delegate {
+                    try {
+                        Process.Start(Platform.LogPath);
+                    } catch (Exception ex) {
+                        Frontend.ShowError(parent, ex);
+                    }
+                });
+            };
+
             Gtk.ComboBox wrapModeComboBox = (Gtk.ComboBox)_Glade["WrapModeComboBox"];
             // initialize wrap modes
             // glade might initialize it already!
@@ -161,13 +183,19 @@ namespace Smuxi.Frontend.Gnome
                                                 Gtk.Stock.Network,
                                                 Gtk.IconSize.SmallToolbar, null),
                             _("Servers"));
-            /*
-            ls.AppendValues(Page.Filters, _Dialog.RenderIcon(
-                                                Gtk.Stock.Delete,
-                                                Gtk.IconSize.SmallToolbar, null),
-                            _("Filters"));
-            */
-            
+
+            if (Frontend.EngineVersion >= new Version("0.7.2")) {
+                // features introduced in >= 0.7.2
+                ls.AppendValues(Page.Filters, _Dialog.RenderIcon(
+                                                    Gtk.Stock.Delete,
+                                                    Gtk.IconSize.SmallToolbar, null),
+                                _("Filters"));
+                ls.AppendValues(Page.Logging, _Dialog.RenderIcon(
+                                                    Gtk.Stock.JustifyLeft,
+                                                    Gtk.IconSize.SmallToolbar, null),
+                                _("Logging"));
+            }
+
             int i = 1;
             _MenuTreeView.AppendColumn(null, new Gtk.CellRendererPixbuf(), "pixbuf",i++);
             _MenuTreeView.AppendColumn(null, new Gtk.CellRendererText(), "text", i++);
@@ -180,9 +208,12 @@ namespace Smuxi.Frontend.Gnome
             ls.GetIterFirst(out iter);
             _MenuTreeView.Selection.SelectIter(iter);
             
-            _ChannelFilterListView = new ChannelFilterListView(_Glade);
-            _ServerListView = new ServerListView(parent, _Glade);
-            
+            _ServerListView = new ServerListView(_Dialog, _Glade);
+            _FilterListWidget = new FilterListWidget(_Dialog, Frontend.UserConfig);
+            _FilterListWidget.Changed += _OnChanged;
+            _FilterListEventBox.Add(_FilterListWidget);
+            _FilterListEventBox.ShowAll();
+
             _Load();
         }
         
@@ -201,7 +232,7 @@ namespace Smuxi.Frontend.Gnome
             ((Gtk.Entry)_Glade["ConnectionRealnameEntry"]).Text  = (string)Frontend.UserConfig["Connection/Realname"];
             string connect_commands = String.Join("\n", (string[])Frontend.UserConfig["Connection/OnConnectCommands"]);
             ((Gtk.TextView)_Glade["OnConnectCommandsTextView"]).Buffer.Text = connect_commands;
-            
+
             string encoding = (string)Frontend.UserConfig["Connection/Encoding"];
             encoding = encoding.ToUpper();
 
@@ -387,6 +418,9 @@ namespace Smuxi.Frontend.Gnome
                 typeof(Gtk.WrapMode),
                 (string) Frontend.UserConfig["Interface/Chat/WrapMode"]
             );
+            if (wrapMode == Gtk.WrapMode.Word) {
+                wrapMode = Gtk.WrapMode.WordChar;
+            }
             int i = 0;
             foreach (object[] row in  (Gtk.ListStore) wrapModeComboBox.Model) {
                 if (((Gtk.WrapMode) row[0]) == wrapMode) {
@@ -406,6 +440,15 @@ namespace Smuxi.Frontend.Gnome
             ((Gtk.SpinButton)_Glade["CommandHistorySizeSpinButton"]).Value =
                 (double)(int)Frontend.UserConfig["Interface/Entry/CommandHistorySize"];
 
+            var highlight_words =
+                (string[]) Frontend.UserConfig["Interface/Chat/HighlightWords"];
+            // backwards compatibility with 0.7.x servers
+            if (highlight_words == null) {
+                highlight_words = new string[] {};
+            }
+            ((Gtk.TextView)_Glade["HighlightWordsTextView"]).Buffer.Text  =
+                    String.Join("\n", highlight_words);
+
             ((Gtk.CheckButton)_Glade["BeepOnHighlightCheckButton"]).Active =
                 (bool)Frontend.UserConfig["Sound/BeepOnHighlight"];
             
@@ -445,11 +488,26 @@ namespace Smuxi.Frontend.Gnome
             }
             
             // Filters
-            _ChannelFilterListView.Load();
+            _FilterListWidget.InitProtocols(Frontend.Session.GetSupportedProtocols());
+            _FilterListWidget.Load();
             
             // Servers
             _ServerListView.Load();
             
+            // Logging
+            ((Gtk.Button) _Glade["LoggingOpenButton"]).Visible = false;
+            if (Frontend.UserConfig["Logging/Enabled"] != null) {
+                ((Gtk.CheckButton) _Glade["LoggingEnabledCheckButton"]).Active =
+                    (bool) Frontend.UserConfig["Logging/Enabled"];
+                if (Frontend.IsLocalEngine) {
+                    ((Gtk.Button) _Glade["LoggingOpenButton"]).Visible = true;
+                }
+            }
+            if (Frontend.UserConfig["Logging/LogFilteredMessages"] != null) {
+                ((Gtk.CheckButton) _Glade["LoggingLogFilteredMessagesCheckButton"]).Active =
+                    (bool) Frontend.UserConfig["Logging/LogFilteredMessages"];
+            }
+
             ((Gtk.Button)_Glade["ApplyButton"]).Sensitive = false;
         }
         
@@ -592,6 +650,26 @@ namespace Smuxi.Frontend.Gnome
             Frontend.UserConfig["Interface/Entry/CommandHistorySize"] =
                 (int)((Gtk.SpinButton)_Glade["CommandHistorySizeSpinButton"]).Value;
             
+            Frontend.UserConfig["Interface/Chat/HighlightWords"] = null;
+
+            string[] highlight_words = ((Gtk.TextView) _Glade["HighlightWordsTextView"]).Buffer.Text.Split(new char[] { '\n' });
+            foreach (string word in highlight_words) {
+                if (word.StartsWith("/") && word.EndsWith("/")) {
+                    try {
+                        Regex regex = new Regex(word.Substring(1, word.Length - 2));
+                    } catch (ArgumentException ex) {
+                        throw new ApplicationException(
+                            String.Format(
+                                _("Invalid highlight regex: '{0}'. Reason: {1}"),
+                                word, ex.Message
+                            )
+                        );
+                    }
+                }
+            }
+
+            Frontend.UserConfig["Interface/Chat/HighlightWords"] = highlight_words;
+
             Frontend.UserConfig["Sound/BeepOnHighlight"] =
                 ((Gtk.CheckButton)_Glade["BeepOnHighlightCheckButton"]).Active;
             
@@ -613,11 +691,18 @@ namespace Smuxi.Frontend.Gnome
             }
             
             // Filters
-            _ChannelFilterListView.Save();
-            
+            _FilterListWidget.Save();
+
             // Servers
+            // _ServerListView saves directly after each change
             //_ServerListView.Save();
             
+            // Logging
+            Frontend.UserConfig["Logging/Enabled"] =
+                ((Gtk.CheckButton) _Glade["LoggingEnabledCheckButton"]).Active;
+            Frontend.UserConfig["Logging/LogFilteredMessages"] =
+                ((Gtk.CheckButton) _Glade["LoggingLogFilteredMessagesCheckButton"]).Active;
+
             Frontend.Config.Save();
         }
         
@@ -641,7 +726,6 @@ namespace Smuxi.Frontend.Gnome
             } catch (Exception ex) {
 #if LOG4NET
                 _Logger.Error(ex);
-                _Logger.Error("BaseException", ex.GetBaseException());
 #endif                
                 Frontend.ShowException(_Dialog, ex);
             }
@@ -657,6 +741,8 @@ namespace Smuxi.Frontend.Gnome
                 Frontend.UserConfig.ClearCache();
                 _Load();
                 Frontend.ApplyConfig(Frontend.UserConfig);
+            } catch (ApplicationException ex) {
+                Frontend.ShowError(_Dialog, ex.Message);
             } catch (Exception ex) {
 #if LOG4NET
                 _Logger.Error(ex);
