@@ -41,6 +41,7 @@ namespace Smuxi.Frontend.Gnome
 #if LOG4NET
         private static readonly log4net.ILog _Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
 #endif
+        public    string             ID { get; private set; }
         private   string             _Name;
         private   ChatModel          _ChatModel;
         private   bool               _HasHighlight;
@@ -206,11 +207,13 @@ namespace Smuxi.Frontend.Gnome
             
             _ChatModel = chat;
             _Name = _ChatModel.Name;
+            ID = _ChatModel.ID;
             Name = _Name;
             
             MessageTextView tv = new MessageTextView();
             _EndMark = tv.Buffer.CreateMark("end", tv.Buffer.EndIter, false); 
             tv.ShowTimestamps = true;
+            tv.ShowMarkerline = true;
             tv.Editable = false;
             tv.CursorVisible = true;
             tv.WrapMode = Gtk.WrapMode.Char;
@@ -368,6 +371,8 @@ namespace Smuxi.Frontend.Gnome
         public virtual void Disable()
         {
             Trace.Call();
+
+            _IsSynced = false;
         }
         
         public virtual void Sync()
@@ -478,14 +483,6 @@ namespace Smuxi.Frontend.Gnome
                 tv.ScrollMarkOnscreen(tv.Buffer.InsertMark);
             }
 
-            int buffer_lines = (int)Frontend.UserConfig["Interface/Notebook/BufferLines"];
-            if (tv.Buffer.LineCount > buffer_lines) {
-                Gtk.TextIter start_iter = tv.Buffer.StartIter;
-                // TODO: maybe we should delete chunks instead of each line
-                Gtk.TextIter end_iter = tv.Buffer.GetIterAtLine(tv.Buffer.LineCount - buffer_lines);
-                tv.Buffer.Delete(ref start_iter, ref end_iter);
-            }
-
             // update the end mark
             tv.Buffer.MoveMark(_EndMark, tv.Buffer.EndIter);
         }
