@@ -41,6 +41,7 @@ namespace Smuxi.Frontend.Gnome
 #if LOG4NET
         private static readonly log4net.ILog _Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
 #endif
+        public static Gdk.Pixbuf   IconPixbuf { get; private set; }
         private GroupChatModel     _GroupChatModel;
         private Gtk.ScrolledWindow _PersonScrolledWindow;
         private Gtk.TreeView       _PersonTreeView;
@@ -51,20 +52,8 @@ namespace Smuxi.Frontend.Gnome
         private Gtk.HPaned         _OutputHPaned;
         private Gtk.ScrolledWindow _TopicScrolledWindow;
         private MessageTextView    _TopicTextView;
+        private MessageModel       _Topic;
         private Gtk.TreeViewColumn _IdentityNameColumn;
-        private Gtk.Image          _TabImage;
-        
-        public Gtk.ScrolledWindow TopicScrolledWindow {
-            get {
-                return _TopicScrolledWindow;
-            }
-        }
-
-        public MessageTextView TopicTextView {
-            get {
-                return _TopicTextView;
-            }
-        }
 
         public override bool HasSelection {
             get {
@@ -81,7 +70,21 @@ namespace Smuxi.Frontend.Gnome
                        _TopicTextView.HasFocus;
             }
         }
-        
+
+        public MessageModel Topic {
+            get {
+                return _Topic;
+            }
+            set {
+                _Topic = value;
+                _TopicTextView.Clear();
+                if (value != null) {
+                    _TopicTextView.AddMessage(value, false);
+                }
+                _TopicScrolledWindow.Visible = !_TopicTextView.IsEmpty;
+            }
+        }
+
         protected Gtk.TreeView PersonTreeView {
             get {
                 return _PersonTreeView;
@@ -100,6 +103,10 @@ namespace Smuxi.Frontend.Gnome
             }
         }
         
+        static GroupChatView() {
+            IconPixbuf = new Gdk.Pixbuf(null, "group-chat.svg", 16, 16);
+       }
+
         public GroupChatView(GroupChatModel groupChat) : base(groupChat)
         {
             Trace.Call(groupChat);
@@ -147,6 +154,7 @@ namespace Smuxi.Frontend.Gnome
             // don't loose the focus else we lose the selection too!
             // see OnPersonTreeViewFocusOutEvent()
             _PersonMenu.TakeFocus = false;
+            _PersonMenu.Shown += OnPersonMenuShown;
             
             _PersonTreeView.ButtonPressEvent += _OnPersonTreeViewButtonPressEvent;
             _PersonTreeView.KeyPressEvent += OnPersonTreeViewKeyPressEvent;
@@ -187,16 +195,8 @@ namespace Smuxi.Frontend.Gnome
             
             ApplyConfig(Frontend.UserConfig);
             
-            _TabImage = new Gtk.Image(
-                new Gdk.Pixbuf(
-                    null,
-                    "group-chat.svg",
-                    16,
-                    16
-                )
-            );
-
-            TabHBox.PackStart(_TabImage, false, false, 2);
+            var tabImage = new Gtk.Image(IconPixbuf);
+            TabHBox.PackStart(tabImage, false, false, 2);
             TabHBox.ShowAll();
             
             ShowAll();
@@ -267,6 +267,8 @@ namespace Smuxi.Frontend.Gnome
                     */
                 }
                 // attach the model again
+                // BUG? TreeView doesn't seem to recognize existing values in the model?!?
+                // see: http://www.smuxi.org/issues/show/132
                 _PersonTreeView.Model = ls;
                 
                 /*
@@ -291,13 +293,8 @@ namespace Smuxi.Frontend.Gnome
             _Logger.Debug("Sync() syncing topic");
 #endif
             // sync topic
-            MessageModel topic = _GroupChatModel.Topic;
-            if ((_TopicTextView != null) &&
-               (topic != null)) {
-                // XXX
-                SetTopic(topic);
-            }
-            
+            Topic = _GroupChatModel.Topic;
+
             base.Sync();
         }
         
@@ -377,82 +374,6 @@ namespace Smuxi.Frontend.Gnome
             UpdatePersonCount();
         }
         
-        public void SetTopic(MessageModel topic)
-        {
-            Trace.Call(topic);
-            Gtk.TextIter iter = _TopicTextView.Buffer.EndIter;
-
-            _TopicTextView.Clear();
-            _TopicTextView.AddMessage(topic, false);
-            _TopicScrolledWindow.Visible = true;
-
-            /*
-            foreach (MessagePartModel topicPart in topic.MessageParts) {
-#if LOG4NET
-                _Logger.Debug("SetTopic(): topicPart.GetType(): " + topicPart.GetType());
-#endif
-                Gdk.Color bgColor = _TopicTextView.DefaultAttributes.Appearance.BgColor;
-//                if (_BackgroundColor != null) {
-//                    bgColor = _BackgroundColor.Value;
-//                }
-                TextColor bgTextColor = ColorTools.GetTextColor(bgColor);
-                // TODO: implement all types
-                if (topicPart is UrlMessagePartModel) {
-                    UrlMessagePartModel fmsgui = (UrlMessagePartModel) topicPart;
-                    // HACK: the engine should set a color for us!
-                    Gtk.TextTag urlTag = _TopicTextTagTable.Lookup("url");
-                    Gdk.Color urlColor = urlTag.ForegroundGdk;
-                    //Console.WriteLine("urlColor: " + urlColor);
-                    TextColor urlTextColor = ColorTools.GetTextColor(urlColor);
-                    urlTextColor = ColorTools.GetBestTextColor(urlTextColor, bgTextColor);
-                    //Console.WriteLine("GetBestTextColor({0}, {1}): {2}",  urlColor, bgTextColor, urlTextColor);
-                    urlTag.ForegroundGdk = ColorTools.GetGdkColor(urlTextColor);
-                    _TopicTextView.Buffer.InsertWithTagsByName(ref iter, fmsgui.Url, "url");
-                } else if (topicPart is TextMessagePartModel) {
-                    TextMessagePartModel fmsgti = (TextMessagePartModel) topicPart;
-#if LOG4NET
-                    _Logger.Debug("SetTopic(): fmsgti.Text: '" + fmsgti.Text + "'");
-#endif
-                    List<string> tags = new List<string>();
-                    if (fmsgti.ForegroundColor != TextColor.None) {
-                        TextColor color = ColorTools.GetBestTextColor(fmsgti.ForegroundColor, bgTextColor);
-                        //Console.WriteLine("GetBestTextColor({0}, {1}): {2}",  fmsgti.ForegroundColor, bgTextColor, color);
-                        string tagname = GetTextTagName(color, null);
-                        //string tagname = _GetTextTagName(fmsgti.ForegroundColor, null);
-                        tags.Add(tagname);
-                    }
-                    if (fmsgti.BackgroundColor != TextColor.None) {
-                        string tagname = GetTextTagName(null, fmsgti.BackgroundColor);
-                        tags.Add(tagname);
-                    }
-                    if (fmsgti.Underline) {
-#if LOG4NET
-                        _Logger.Debug("SetTopic(): fmsgti.Underline is true");
-#endif
-                        tags.Add("underline");
-                    }
-                    if (fmsgti.Bold) {
-#if LOG4NET
-                        _Logger.Debug("SetTopic(): fmsgti.Bold is true");
-#endif
-                        tags.Add("bold");
-                    }
-                    if (fmsgti.Italic) {
-#if LOG4NET
-                        _Logger.Debug("SetTopic(): fmsgti.Italic is true");
-#endif
-                        tags.Add("italic");
-                    }
-                    
-                    _TopicTextView.Buffer.InsertWithTagsByName(ref iter,
-                                                               fmsgti.Text,
-                                                               tags.ToArray());
-                } 
-            }
-            _TopicTextView.Buffer.Insert(ref iter, "\n");
-            */
-        }
-
         public override void ApplyConfig(UserConfig config)
         {
             Trace.Call(config);
@@ -596,6 +517,10 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        protected virtual void OnPersonMenuShown(object sender, EventArgs e)
+        {
+        }
+
         private void _OnUserListButtonReleaseEvent(object sender, Gtk.ButtonReleaseEventArgs e)
         {
             Trace.Call(sender, e);
