@@ -44,10 +44,14 @@ namespace Smuxi.Frontend.Gnome
         private MessageModel _LastMessage;
         private bool         _ShowTimestamps;
         private bool         _ShowHighlight;
+        private bool         _ShowMarkerline;
         private bool         _AtUrlTag;
         private string       _Url;
         private UserConfig   _Config;
         private ThemeSettings _ThemeSettings;
+        private Gdk.Color    _MarkerlineColor = new Gdk.Color(255, 0, 0);
+        private int          _MarkerlineBufferPosition;
+        private int          _BufferLines = -1;
 
         public event MessageTextViewMessageAddedEventHandler       MessageAdded;
         public event MessageTextViewMessageHighlightedEventHandler MessageHighlighted;
@@ -70,6 +74,15 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        public bool ShowMarkerline {
+            get {
+                return _ShowMarkerline;
+            }
+            set {
+                _ShowMarkerline = value;
+            }
+        }
+
         public bool HasTextViewSelection {
             get {
 #if GTK_SHARP_2_10
@@ -88,6 +101,24 @@ namespace Smuxi.Frontend.Gnome
             }
         }
 
+        public bool IsEmpty {
+            get {
+                return Buffer.CharCount == 0;
+            }
+        }
+
+        public bool IsAtUrlTag {
+            get {
+                return _AtUrlTag;
+            }
+        }
+
+        public MessageModel LastMessage {
+            get {
+                return _LastMessage;
+            }
+        }
+
         public MessageTextView()
         {
             Trace.Call();
@@ -98,6 +129,7 @@ namespace Smuxi.Frontend.Gnome
             Buffer = new Gtk.TextBuffer(_MessageTextTagTable);
             MotionNotifyEvent += OnMotionNotifyEvent;
             PopulatePopup += OnPopulatePopup;
+            ExposeEvent += OnExposeEvent;
         }
 
         public void ApplyConfig(UserConfig config)
@@ -126,6 +158,8 @@ namespace Smuxi.Frontend.Gnome
                 }
                 WrapMode = wrapMode;
             }
+
+            _BufferLines = (int) config["Interface/Notebook/BufferLines"];
         }
 
         public void Clear()
@@ -137,15 +171,17 @@ namespace Smuxi.Frontend.Gnome
 
         public void AddMessage(MessageModel msg)
         {
-            Trace.Call(msg);
-            
             AddMessage(msg, true);
         }
         
         public void AddMessage(MessageModel msg, bool addLinebreak)
         {
             Trace.Call(msg, addLinebreak);
-            
+
+            if (msg == null) {
+                throw new ArgumentNullException("msg");
+            }
+
             Gtk.TextIter iter = Buffer.EndIter;
 
             Gdk.Color bgColor = DefaultAttributes.Appearance.BgColor;
@@ -265,6 +301,8 @@ namespace Smuxi.Frontend.Gnome
                 Buffer.Insert(ref iter, "\n");
             }
 
+            CheckBufferSize();
+
             // HACK: force a redraw of the widget, as for some reason
             // GTK+ 2.17.6 is not redrawing some lines we add here, especially
             // for local messages. See:
@@ -284,6 +322,18 @@ namespace Smuxi.Frontend.Gnome
             _LastMessage = msg;
         }
 
+        public void UpdateMarkerline()
+        {
+            Trace.Call();
+
+            if (IsEmpty) {
+                return;
+            }
+
+            _MarkerlineBufferPosition = Buffer.EndIter.Offset - 1;
+            QueueDraw();
+        }
+
         /*
          * Helper methods
          */
@@ -495,6 +545,59 @@ namespace Smuxi.Frontend.Gnome
              return tagname;
         }
         
+        void OnExposeEvent(object sender, Gtk.ExposeEventArgs e)
+        {
+            if (!_ShowMarkerline || _MarkerlineBufferPosition == 0) {
+                return;
+            }
+
+            var window = e.Event.Window;
+            var gc = new Gdk.GC(window);
+            gc.RgbFgColor = _MarkerlineColor;
+
+            var iter = Buffer.GetIterAtOffset(_MarkerlineBufferPosition);
+            var location = GetIterLocation(iter);
+            int last_y = location.Y + location.Height;
+            // padding
+            last_y += PixelsAboveLines + PixelsBelowLines / 2;
+
+            int x, y;
+            BufferToWindowCoords(Gtk.TextWindowType.Text, 0,
+                                 last_y, out x, out y);
+
+            if (y < e.Event.Area.Y) {
+                return;
+            }
+
+            window.DrawLine(gc, 0, y, VisibleRect.Width, y);
+        }
+
+        void CheckBufferSize()
+        {
+            if (_BufferLines == -1) {
+                // no limit defined
+                return;
+            }
+
+            if (Buffer.LineCount > _BufferLines) {
+                Gtk.TextIter start_iter = Buffer.StartIter;
+                // TODO: maybe we should delete chunks instead of each line
+                Gtk.TextIter end_iter = Buffer.GetIterAtLine(Buffer.LineCount -
+                                                             _BufferLines);
+                int offset = end_iter.Offset;
+                Buffer.Delete(ref start_iter, ref end_iter);
+
+                // update markerline offset if present
+                if (_MarkerlineBufferPosition != 0) {
+                    _MarkerlineBufferPosition -= offset;
+                    // remove markerline if it went out of buffer
+                    if (_MarkerlineBufferPosition < 0) {
+                        _MarkerlineBufferPosition = 0;
+                    }
+                }
+            }
+        }
+
         private static string _(string msg)
         {
             return Mono.Unix.Catalog.GetString(msg);
