@@ -131,6 +131,8 @@ namespace Smuxi.Frontend
         
         public void Setup()
         {
+            Trace.Call();
+
             if (String.IsNullOrEmpty(f_Program)) {
                 // use plink by default if it's there
                 if (File.Exists("plink.exe")) {
@@ -144,7 +146,7 @@ namespace Smuxi.Frontend
                 throw new ApplicationException(_("SSH client application was not found: " + f_Program));
             }
             if (f_Program.ToLower().EndsWith("putty.exe")) {
-                throw new ApplicationException(_("SSH client must be either OpenSSH (ssh) or Plink (plink.exe, _not_ putty.exe)"));
+                throw new ApplicationException(_("SSH client must be either OpenSSH (ssh) or Plink (plink.exe, not putty.exe)"));
             }
             
             bool isPutty = false;
@@ -171,6 +173,25 @@ namespace Smuxi.Frontend
         
         public void Connect()
         {
+            Trace.Call();
+
+#if LOG4NET
+            f_Logger.Debug("Connect(): checking if local forward port is free...");
+#endif
+            using (TcpClient tcpClient = new TcpClient()) {
+                try {
+                    tcpClient.Connect(f_ForwardBindAddress, f_ForwardBindPort);
+                    // the connect worked, panic!
+                    var msg = String.Format(
+                        _("The local SSH forwarding port {0} is already in " +
+                          "use. Is there an old SSH tunnel still active?"),
+                        f_ForwardBindPort
+                    );
+                    throw new ApplicationException(msg);
+                } catch (SocketException ex) {
+                }
+            }
+
 #if LOG4NET
             f_Logger.Debug("Connect(): setting up ssh tunnel using command: " +
                            f_ProcessStartInfo.FileName + " " +
@@ -186,12 +207,12 @@ namespace Smuxi.Frontend
                     string output = f_Process.StandardOutput.ReadToEnd();
                     string error = f_Process.StandardError.ReadToEnd();
                     string msg = String.Format(
-                        _("SSH tunnel setup failed with (exit code: {0})\n\n" +
+                        _("SSH tunnel setup failed (exit code: {0})\n\n" +
                           "SSH program: {1}\n" +
                           "SSH parameters: {2}\n\n" +
                           "Program Error:\n" +
                           "{3}\n" +
-                          "Prgram Output:\n" +
+                          "Program Output:\n" +
                           "{4}\n"),
                         f_Process.ExitCode,
                         f_ProcessStartInfo.FileName,
@@ -251,8 +272,17 @@ namespace Smuxi.Frontend
         
         public void Disconnect()
         {
+            Trace.Call();
+
             if (f_Process != null && !f_Process.HasExited) {
+#if LOG4NET
+                f_Logger.Debug("Disconnect(): killing ssh tunnel...");
+#endif
                 f_Process.Kill();
+                f_Process.WaitForExit();
+#if LOG4NET
+                f_Logger.Debug("Disconnect(): ssh tunnel exited");
+#endif
             }
         }
         
@@ -260,16 +290,25 @@ namespace Smuxi.Frontend
         {
             string sshArguments = String.Empty;
 
+            Version sshVersion = GetOpenSshVersion();
             // starting with OpenSSH version 4.4p1 we can use the
             // ExitOnForwardFailure option for detecting tunnel issues better
             // as the process will quit nicely, for more details see:
             // http://projects.qnetp.net/issues/show/145
             // NOTE: the patch level is mapped to the micro component
-            if (GetOpenSshVersion() >= new Version("4.4.1")) {
+            if (sshVersion >= new Version("4.4.1")) {
                 // exit if the tunnel setup didn't work somehow
                 sshArguments += " -o ExitOnForwardFailure=yes";
             }
-            
+
+            // with OpenSSH 3.8 we can use the keep-alive feature of SSH that
+            // will check the remote peer in defined intervals and kills the
+            // tunnel if it reached the max value
+            if (sshVersion >= new Version("3.8")) {
+                // exit if the peer can't be reached for more than 90 seconds
+                sshArguments += " -o ServerAliveInterval=30 -o ServerAliveCountMax=3";
+            }
+
             // run in the background (detach)
             // plink doesn't support this and we can't control the process this way!
             //sshArguments += " -f";
@@ -363,11 +402,11 @@ namespace Smuxi.Frontend
             }
 
             string msg = String.Format(
-                _("Couldn't get OpenSSH version (exit code: {0})\n\n" +
+                _("OpenSSH version number not found (exit code: {0})\n\n" +
                   "SSH program: {1}\n\n" +
                   "Program Error:\n" +
                   "{2}\n" +
-                  "Prgram Output:\n" +
+                  "Program Output:\n" +
                   "{3}\n"),
                 f_Process.ExitCode,
                 f_Program,
